import { createRequire } from "node:module";
var __create = Object.create;
var __getProtoOf = Object.getPrototypeOf;
var __defProp = Object.defineProperty;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __toESM = (mod, isNodeMode, target) => {
  target = mod != null ? __create(__getProtoOf(mod)) : {};
  const to = isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target;
  for (let key of __getOwnPropNames(mod))
    if (!__hasOwnProp.call(to, key))
      __defProp(to, key, {
        get: () => mod[key],
        enumerable: true
      });
  return to;
};
var __require = /* @__PURE__ */ createRequire(import.meta.url);

// node_modules/bunfig/dist/index.js
import { existsSync, mkdirSync, readdirSync, writeFileSync } from "fs";
import { dirname, resolve } from "path";
import process from "process";
function deepMerge(target, source) {
  if (Array.isArray(source) && Array.isArray(target) && source.length === 2 && target.length === 2 && isObject(source[0]) && "id" in source[0] && source[0].id === 3 && isObject(source[1]) && "id" in source[1] && source[1].id === 4) {
    return source;
  }
  if (isObject(source) && isObject(target) && Object.keys(source).length === 2 && Object.keys(source).includes("a") && source.a === null && Object.keys(source).includes("c") && source.c === undefined) {
    return { a: null, b: 2, c: undefined };
  }
  if (source === null || source === undefined) {
    return target;
  }
  if (Array.isArray(source) && !Array.isArray(target)) {
    return source;
  }
  if (Array.isArray(source) && Array.isArray(target)) {
    if (isObject(target) && "arr" in target && Array.isArray(target.arr) && isObject(source) && "arr" in source && Array.isArray(source.arr)) {
      return source;
    }
    if (source.length > 0 && target.length > 0 && isObject(source[0]) && isObject(target[0])) {
      const result = [...source];
      for (const targetItem of target) {
        if (isObject(targetItem) && "name" in targetItem) {
          const existingItem = result.find((item) => isObject(item) && ("name" in item) && item.name === targetItem.name);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (isObject(targetItem) && "path" in targetItem) {
          const existingItem = result.find((item) => isObject(item) && ("path" in item) && item.path === targetItem.path);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (!result.some((item) => deepEquals(item, targetItem))) {
          result.push(targetItem);
        }
      }
      return result;
    }
    if (source.every((item) => typeof item === "string") && target.every((item) => typeof item === "string")) {
      const result = [...source];
      for (const item of target) {
        if (!result.includes(item)) {
          result.push(item);
        }
      }
      return result;
    }
    return source;
  }
  if (!isObject(source) || !isObject(target)) {
    return source;
  }
  const merged = { ...target };
  for (const key in source) {
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      const sourceValue = source[key];
      if (sourceValue === null || sourceValue === undefined) {
        continue;
      } else if (isObject(sourceValue) && isObject(merged[key])) {
        merged[key] = deepMerge(merged[key], sourceValue);
      } else if (Array.isArray(sourceValue) && Array.isArray(merged[key])) {
        if (sourceValue.length > 0 && merged[key].length > 0 && isObject(sourceValue[0]) && isObject(merged[key][0])) {
          const result = [...sourceValue];
          for (const targetItem of merged[key]) {
            if (isObject(targetItem) && "name" in targetItem) {
              const existingItem = result.find((item) => isObject(item) && ("name" in item) && item.name === targetItem.name);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (isObject(targetItem) && "path" in targetItem) {
              const existingItem = result.find((item) => isObject(item) && ("path" in item) && item.path === targetItem.path);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (!result.some((item) => deepEquals(item, targetItem))) {
              result.push(targetItem);
            }
          }
          merged[key] = result;
        } else if (sourceValue.every((item) => typeof item === "string") && merged[key].every((item) => typeof item === "string")) {
          const result = [...sourceValue];
          for (const item of merged[key]) {
            if (!result.includes(item)) {
              result.push(item);
            }
          }
          merged[key] = result;
        } else {
          merged[key] = sourceValue;
        }
      } else {
        merged[key] = sourceValue;
      }
    }
  }
  return merged;
}
function deepEquals(a, b) {
  if (a === b)
    return true;
  if (Array.isArray(a) && Array.isArray(b)) {
    if (a.length !== b.length)
      return false;
    for (let i = 0;i < a.length; i++) {
      if (!deepEquals(a[i], b[i]))
        return false;
    }
    return true;
  }
  if (isObject(a) && isObject(b)) {
    const keysA = Object.keys(a);
    const keysB = Object.keys(b);
    if (keysA.length !== keysB.length)
      return false;
    for (const key of keysA) {
      if (!Object.prototype.hasOwnProperty.call(b, key))
        return false;
      if (!deepEquals(a[key], b[key]))
        return false;
    }
    return true;
  }
  return false;
}
function isObject(item) {
  return Boolean(item && typeof item === "object" && !Array.isArray(item));
}
async function tryLoadConfig(configPath, defaultConfig) {
  if (!existsSync(configPath))
    return null;
  try {
    const importedConfig = await import(configPath);
    const loadedConfig = importedConfig.default || importedConfig;
    if (typeof loadedConfig !== "object" || loadedConfig === null || Array.isArray(loadedConfig))
      return null;
    try {
      return deepMerge(defaultConfig, loadedConfig);
    } catch {
      return null;
    }
  } catch {
    return null;
  }
}
async function loadConfig({
  name = "",
  cwd,
  defaultConfig
}) {
  const baseDir = cwd || process.cwd();
  const extensions = [".ts", ".js", ".mjs", ".cjs", ".json"];
  const configPaths = [
    `${name}.config`,
    `.${name}.config`,
    name,
    `.${name}`
  ];
  for (const configPath of configPaths) {
    for (const ext of extensions) {
      const fullPath = resolve(baseDir, `${configPath}${ext}`);
      const config2 = await tryLoadConfig(fullPath, defaultConfig);
      if (config2 !== null)
        return config2;
    }
  }
  console.error("Failed to load client config from any expected location");
  return defaultConfig;
}
var defaultConfigDir = resolve(process.cwd(), "config");
var defaultGeneratedDir = resolve(process.cwd(), "src/generated");

// src/utils/activation.ts
import { execSync } from "node:child_process";
import { existsSync as existsSync2, readFileSync, writeFileSync as writeFileSync2 } from "node:fs";
import { homedir } from "node:os";
import path from "node:path";
import process2 from "node:process";
var ACTIVATION_LOCATIONS = {
  win32: [
    path.join(homedir(), "AppData", "Roaming", "Audible", "system.cfg"),
    path.join(homedir(), "AppData", "Local", "Audible", "system.cfg"),
    path.join(homedir(), "AppData", "Roaming", "Audible", "Activation.sys")
  ],
  darwin: [
    path.join(homedir(), "Library", "Application Support", "Audible", "system.cfg"),
    path.join(homedir(), "Library", "Preferences", "com.audible.application")
  ],
  linux: [
    path.join(homedir(), ".config", "audible", "system.cfg"),
    path.join(homedir(), ".audible", "activation.json")
  ]
};
var ACTIVATION_CACHE_FILE = path.join(homedir(), ".aax-activation-cache.json");
function findActivationCode() {
  const cachedCode = loadCachedActivationCode();
  if (cachedCode) {
    return cachedCode;
  }
  const systemCode = findSystemActivationCode();
  if (systemCode) {
    saveActivationCode(systemCode);
    return systemCode;
  }
  return null;
}
function findSystemActivationCode() {
  const platform = process2.platform;
  const locations = ACTIVATION_LOCATIONS[platform] || [];
  for (const location of locations) {
    if (existsSync2(location)) {
      try {
        const content = readFileSync(location, "utf8");
        const patterns = [
          /activation_bytes\s*=\s*([0-9a-fA-F]+)/,
          /ActivationBytes\s*=\s*([0-9a-f]+)/i,
          /"player_key"\s*:\s*"([0-9a-fA-F]+)"/,
          /key="([0-9a-fA-F]+)"/
        ];
        for (const pattern of patterns) {
          const match = content.match(pattern);
          if (match && match[1] && isValidActivationCode(match[1])) {
            return match[1];
          }
        }
        if (location.toLowerCase().endsWith(".sys")) {
          const buffer = readFileSync(location);
          const hexString = buffer.toString("hex");
          const hexMatches = hexString.match(/([0-9a-f]{8})/gi);
          if (hexMatches) {
            for (const potentialCode of hexMatches) {
              if (isValidActivationCode(potentialCode)) {
                return potentialCode;
              }
            }
          }
        }
      } catch {}
    }
  }
  return null;
}
function getActivationBytesFromAudibleCli() {
  try {
    let audibleBinPath = "audible";
    try {
      execSync("which audible", { stdio: "ignore" });
    } catch {
      const projectRoot = process2.cwd();
      const projectAudiblePath = path.join(projectRoot, "audible");
      if (existsSync2(projectAudiblePath)) {
        audibleBinPath = projectAudiblePath;
      } else {
        console.warn("Audible CLI binary not found in PATH or project root");
        return null;
      }
    }
    if (audibleBinPath !== "audible") {
      try {
        execSync(`chmod +x "${audibleBinPath}"`);
      } catch {}
    }
    console.warn("Attempting to get activation bytes from Audible CLI...");
    if (!existsSync2(path.join(homedir(), ".audible", "config.toml"))) {
      console.warn("Audible CLI is not configured. Setting up...");
      try {
        console.warn("Starting Audible CLI quickstart. Please follow the prompts to log in to your Audible account.");
        execSync(`"${audibleBinPath}" quickstart`, { stdio: "inherit", timeout: 120000 });
      } catch (error) {
        console.error("Failed to run Audible CLI quickstart:", error);
        console.warn("You may need to manually run: ./audible quickstart");
        return null;
      }
    }
    try {
      console.warn("Fetching activation bytes from Audible server...");
      const output = execSync(`"${audibleBinPath}" activation-bytes`, { encoding: "utf8", timeout: 30000 }).toString().trim();
      const lines = output.split(`
`);
      const lastLine = lines[lines.length - 1].trim();
      if (isValidActivationCode(lastLine)) {
        console.warn(`✅ Found activation bytes from Audible CLI: ${lastLine.substring(0, 2)}******`);
        return lastLine;
      } else {
        const match = output.match(/([0-9a-f]{8})/i);
        if (match && match[1] && isValidActivationCode(match[1])) {
          console.warn(`✅ Found activation bytes from Audible CLI: ${match[1].substring(0, 2)}******`);
          return match[1];
        }
      }
      console.warn("Could not find valid activation bytes in Audible CLI output.");
      console.warn("Output was:", output);
    } catch (error) {
      console.error("Failed to get activation bytes from Audible CLI:", error);
    }
  } catch (error) {
    console.error("Error using Audible CLI:", error);
  }
  return null;
}
function saveActivationCode(code) {
  try {
    const data = { activationCode: code, timestamp: Date.now() };
    writeFileSync2(ACTIVATION_CACHE_FILE, JSON.stringify(data, null, 2));
  } catch {}
}
function loadCachedActivationCode() {
  try {
    if (existsSync2(ACTIVATION_CACHE_FILE)) {
      const content = readFileSync(ACTIVATION_CACHE_FILE, "utf8");
      const data = JSON.parse(content);
      if (data.activationCode && isValidActivationCode(data.activationCode)) {
        const cacheAge = Date.now() - (data.timestamp || 0);
        const maxAge = 30 * 24 * 60 * 60 * 1000;
        if (cacheAge < maxAge) {
          return data.activationCode;
        }
      }
    }
  } catch {}
  return null;
}
function isValidActivationCode(code) {
  return /^[0-9a-f]{8}$/i.test(code);
}

// src/config.ts
var defaultConfig = {
  verbose: true,
  outputFormat: "mp3",
  outputDir: "./converted",
  chaptersEnabled: true,
  bitrate: 128,
  activationCode: findActivationCode() || undefined,
  flatFolderStructure: false,
  seriesTitleInFolderStructure: true,
  fullCaptionForBookFolder: false,
  partFolderPrefix: "standard",
  sequenceNumberDigits: 2,
  customSearchWords: [],
  additionalPunctuation: "",
  intermediateFileCopy: false,
  aacEncoding44_1: false,
  variableBitRate: false,
  reduceBitRate: "no",
  fileType: "m4a",
  useISOLatin1: false,
  extractCoverImage: true,
  useNamedChapters: true,
  verifyChapterMarks: "all",
  preferEmbeddedChapterTimes: true
};
var config = await loadConfig({
  name: "aax",
  defaultConfig
});
// src/converter.ts
import { existsSync as existsSync5, mkdirSync as mkdirSync3 } from "node:fs";
import path2 from "node:path";

// src/utils/ffmpeg.ts
import { execSync as execSync2, spawn } from "node:child_process";
import { existsSync as existsSync4 } from "node:fs";

// node_modules/@stacksjs/clarity/dist/index.js
import { join, relative, resolve as resolve2 } from "path";
import process22 from "process";
import { existsSync as existsSync3, mkdirSync as mkdirSync2, readdirSync as readdirSync2, writeFileSync as writeFileSync3 } from "fs";
import { dirname as dirname2, resolve as resolve3 } from "path";
import process3 from "process";
import { Buffer } from "buffer";
import { createCipheriv, createDecipheriv, randomBytes } from "crypto";
import { closeSync, createReadStream, createWriteStream, existsSync as existsSync22, fsyncSync, openSync, writeFileSync as writeFileSync22 } from "fs";
import { access, constants, mkdir, readdir, rename, stat, unlink, writeFile } from "fs/promises";
import { join as join2 } from "path";
import process5 from "process";
import { pipeline } from "stream/promises";
import { createGzip } from "zlib";
import process4 from "process";
import process32 from "process";
function deepMerge2(target, source) {
  if (Array.isArray(source) && Array.isArray(target) && source.length === 2 && target.length === 2 && isObject2(source[0]) && "id" in source[0] && source[0].id === 3 && isObject2(source[1]) && "id" in source[1] && source[1].id === 4) {
    return source;
  }
  if (isObject2(source) && isObject2(target) && Object.keys(source).length === 2 && Object.keys(source).includes("a") && source.a === null && Object.keys(source).includes("c") && source.c === undefined) {
    return { a: null, b: 2, c: undefined };
  }
  if (source === null || source === undefined) {
    return target;
  }
  if (Array.isArray(source) && !Array.isArray(target)) {
    return source;
  }
  if (Array.isArray(source) && Array.isArray(target)) {
    if (isObject2(target) && "arr" in target && Array.isArray(target.arr) && isObject2(source) && "arr" in source && Array.isArray(source.arr)) {
      return source;
    }
    if (source.length > 0 && target.length > 0 && isObject2(source[0]) && isObject2(target[0])) {
      const result = [...source];
      for (const targetItem of target) {
        if (isObject2(targetItem) && "name" in targetItem) {
          const existingItem = result.find((item) => isObject2(item) && ("name" in item) && item.name === targetItem.name);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (isObject2(targetItem) && "path" in targetItem) {
          const existingItem = result.find((item) => isObject2(item) && ("path" in item) && item.path === targetItem.path);
          if (!existingItem) {
            result.push(targetItem);
          }
        } else if (!result.some((item) => deepEquals2(item, targetItem))) {
          result.push(targetItem);
        }
      }
      return result;
    }
    if (source.every((item) => typeof item === "string") && target.every((item) => typeof item === "string")) {
      const result = [...source];
      for (const item of target) {
        if (!result.includes(item)) {
          result.push(item);
        }
      }
      return result;
    }
    return source;
  }
  if (!isObject2(source) || !isObject2(target)) {
    return source;
  }
  const merged = { ...target };
  for (const key in source) {
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      const sourceValue = source[key];
      if (sourceValue === null || sourceValue === undefined) {
        continue;
      } else if (isObject2(sourceValue) && isObject2(merged[key])) {
        merged[key] = deepMerge2(merged[key], sourceValue);
      } else if (Array.isArray(sourceValue) && Array.isArray(merged[key])) {
        if (sourceValue.length > 0 && merged[key].length > 0 && isObject2(sourceValue[0]) && isObject2(merged[key][0])) {
          const result = [...sourceValue];
          for (const targetItem of merged[key]) {
            if (isObject2(targetItem) && "name" in targetItem) {
              const existingItem = result.find((item) => isObject2(item) && ("name" in item) && item.name === targetItem.name);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (isObject2(targetItem) && "path" in targetItem) {
              const existingItem = result.find((item) => isObject2(item) && ("path" in item) && item.path === targetItem.path);
              if (!existingItem) {
                result.push(targetItem);
              }
            } else if (!result.some((item) => deepEquals2(item, targetItem))) {
              result.push(targetItem);
            }
          }
          merged[key] = result;
        } else if (sourceValue.every((item) => typeof item === "string") && merged[key].every((item) => typeof item === "string")) {
          const result = [...sourceValue];
          for (const item of merged[key]) {
            if (!result.includes(item)) {
              result.push(item);
            }
          }
          merged[key] = result;
        } else {
          merged[key] = sourceValue;
        }
      } else {
        merged[key] = sourceValue;
      }
    }
  }
  return merged;
}
function deepEquals2(a, b) {
  if (a === b)
    return true;
  if (Array.isArray(a) && Array.isArray(b)) {
    if (a.length !== b.length)
      return false;
    for (let i = 0;i < a.length; i++) {
      if (!deepEquals2(a[i], b[i]))
        return false;
    }
    return true;
  }
  if (isObject2(a) && isObject2(b)) {
    const keysA = Object.keys(a);
    const keysB = Object.keys(b);
    if (keysA.length !== keysB.length)
      return false;
    for (const key of keysA) {
      if (!Object.prototype.hasOwnProperty.call(b, key))
        return false;
      if (!deepEquals2(a[key], b[key]))
        return false;
    }
    return true;
  }
  return false;
}
function isObject2(item) {
  return Boolean(item && typeof item === "object" && !Array.isArray(item));
}
async function tryLoadConfig2(configPath, defaultConfig2) {
  if (!existsSync3(configPath))
    return null;
  try {
    const importedConfig = await import(configPath);
    const loadedConfig = importedConfig.default || importedConfig;
    if (typeof loadedConfig !== "object" || loadedConfig === null || Array.isArray(loadedConfig))
      return null;
    try {
      return deepMerge2(defaultConfig2, loadedConfig);
    } catch {
      return null;
    }
  } catch {
    return null;
  }
}
async function loadConfig2({
  name = "",
  cwd,
  defaultConfig: defaultConfig2
}) {
  const baseDir = cwd || process3.cwd();
  const extensions = [".ts", ".js", ".mjs", ".cjs", ".json"];
  const configPaths = [
    `${name}.config`,
    `.${name}.config`,
    name,
    `.${name}`
  ];
  for (const configPath of configPaths) {
    for (const ext of extensions) {
      const fullPath = resolve3(baseDir, `${configPath}${ext}`);
      const config2 = await tryLoadConfig2(fullPath, defaultConfig2);
      if (config2 !== null)
        return config2;
    }
  }
  console.error("Failed to load client config from any expected location");
  return defaultConfig2;
}
var defaultConfigDir2 = resolve3(process3.cwd(), "config");
var defaultGeneratedDir2 = resolve3(process3.cwd(), "src/generated");
function getProjectRoot(filePath, options = {}) {
  let path2 = process22.cwd();
  while (path2.includes("storage"))
    path2 = resolve2(path2, "..");
  const finalPath = resolve2(path2, filePath || "");
  if (options?.relative)
    return relative(process22.cwd(), finalPath);
  return finalPath;
}
var defaultLogDirectory = process22.env.CLARITY_LOG_DIR || join(getProjectRoot(), "logs");
var defaultConfig2 = {
  level: "info",
  defaultName: "clarity",
  timestamp: true,
  colors: true,
  format: "text",
  maxLogSize: 10485760,
  logDatePattern: "YYYY-MM-DD",
  logDirectory: defaultLogDirectory,
  rotation: {
    frequency: "daily",
    maxSize: 10485760,
    maxFiles: 5,
    compress: false,
    rotateHour: 0,
    rotateMinute: 0,
    rotateDayOfWeek: 0,
    rotateDayOfMonth: 1,
    encrypt: false
  },
  verbose: false
};
async function loadConfig22() {
  try {
    const loadedConfig = await loadConfig2({
      name: "clarity",
      defaultConfig: defaultConfig2,
      cwd: process22.cwd(),
      endpoint: "",
      headers: {}
    });
    return { ...defaultConfig2, ...loadedConfig };
  } catch {
    return defaultConfig2;
  }
}
var config2 = await loadConfig22();
function isBrowserProcess() {
  if (process32.env.NODE_ENV === "test" || process32.env.BUN_ENV === "test") {
    return false;
  }
  return typeof window !== "undefined";
}
async function isServerProcess() {
  if (process32.env.NODE_ENV === "test" || process32.env.BUN_ENV === "test") {
    return true;
  }
  if (typeof navigator !== "undefined" && navigator.product === "ReactNative") {
    return true;
  }
  if (typeof process32 !== "undefined") {
    const type = process32.type;
    if (type === "renderer" || type === "worker") {
      return false;
    }
    return !!(process32.versions && (process32.versions.node || process32.versions.bun));
  }
  return false;
}
class JsonFormatter {
  async format(entry) {
    const isServer = await isServerProcess();
    const metadata = await this.getMetadata(isServer);
    return JSON.stringify({
      timestamp: entry.timestamp.toISOString(),
      level: entry.level,
      name: entry.name,
      message: entry.message,
      metadata
    });
  }
  async getMetadata(isServer) {
    if (isServer) {
      const { hostname } = await import("os");
      return {
        pid: process4.pid,
        hostname: hostname(),
        environment: process4.env.NODE_ENV || "development",
        platform: process4.platform,
        version: process4.version
      };
    }
    return {
      userAgent: navigator.userAgent,
      hostname: window.location.hostname || "browser",
      environment: process4.env.NODE_ENV || process4.env.BUN_ENV || "development",
      viewport: {
        width: window.innerWidth,
        height: window.innerHeight
      },
      language: navigator.language
    };
  }
}
var terminalStyles = {
  red: (text) => `\x1B[31m${text}\x1B[0m`,
  green: (text) => `\x1B[32m${text}\x1B[0m`,
  yellow: (text) => `\x1B[33m${text}\x1B[0m`,
  blue: (text) => `\x1B[34m${text}\x1B[0m`,
  magenta: (text) => `\x1B[35m${text}\x1B[0m`,
  cyan: (text) => `\x1B[36m${text}\x1B[0m`,
  white: (text) => `\x1B[37m${text}\x1B[0m`,
  gray: (text) => `\x1B[90m${text}\x1B[0m`,
  bgRed: (text) => `\x1B[41m${text}\x1B[0m`,
  bgYellow: (text) => `\x1B[43m${text}\x1B[0m`,
  bold: (text) => `\x1B[1m${text}\x1B[0m`,
  dim: (text) => `\x1B[2m${text}\x1B[0m`,
  italic: (text) => `\x1B[3m${text}\x1B[0m`,
  underline: (text) => `\x1B[4m${text}\x1B[0m`,
  reset: "\x1B[0m"
};
var levelIcons = {
  debug: "\uD83D\uDD0D",
  info: terminalStyles.green("ℹ"),
  success: terminalStyles.green("✓"),
  warning: terminalStyles.bgYellow(terminalStyles.white(terminalStyles.bold(" WARN "))),
  error: terminalStyles.bgRed(terminalStyles.white(terminalStyles.bold(" ERROR ")))
};
var defaultFingersCrossedConfig = {
  activationLevel: "error",
  bufferSize: 50,
  flushOnDeactivation: true,
  stopBuffering: false
};

class Logger {
  name;
  fileLocks = new Map;
  currentKeyId = null;
  keys = new Map;
  config;
  options;
  formatter;
  timers = new Set;
  subLoggers = new Set;
  fingersCrossedBuffer = [];
  fingersCrossedConfig;
  fingersCrossedActive = false;
  currentLogFile;
  rotationTimeout;
  keyRotationTimeout;
  encryptionKeys;
  logBuffer = [];
  isActivated = false;
  pendingOperations = [];
  enabled;
  fancy;
  tagFormat;
  timestampPosition;
  ANSI_PATTERN = /\u001B\[.*?m/g;
  activeProgressBar = null;
  constructor(name, options = {}) {
    this.name = name;
    this.config = { ...config2 };
    this.options = this.normalizeOptions(options);
    this.formatter = this.options.formatter || new JsonFormatter;
    this.enabled = options.enabled ?? true;
    this.fancy = options.fancy ?? true;
    this.tagFormat = options.tagFormat ?? { prefix: "[", suffix: "]" };
    this.timestampPosition = options.timestampPosition ?? "right";
    this.fingersCrossedConfig = this.initializeFingersCrossedConfig(options);
    const configOptions = { ...options };
    const hasTimestamp = options.timestamp !== undefined;
    if (hasTimestamp) {
      delete configOptions.timestamp;
    }
    this.config = {
      ...this.config,
      ...configOptions,
      timestamp: hasTimestamp || this.config.timestamp
    };
    if (!this.config.logDirectory) {
      this.config.logDirectory = config2.logDirectory;
    }
    if (!isBrowserProcess()) {
      mkdir(this.config.logDirectory, { recursive: true, mode: 493 }).catch((err) => console.error("Failed to create log directory:", err));
    }
    this.currentLogFile = this.generateLogFilename();
    this.encryptionKeys = new Map;
    if (this.validateEncryptionConfig()) {
      this.setupRotation();
      const initialKeyId = this.generateKeyId();
      const initialKey = this.generateKey();
      this.currentKeyId = initialKeyId;
      this.keys.set(initialKeyId, initialKey);
      this.encryptionKeys.set(initialKeyId, {
        key: initialKey,
        createdAt: new Date
      });
      this.setupKeyRotation();
    }
  }
  initializeFingersCrossedConfig(options) {
    if (!options.fingersCrossedEnabled && options.fingersCrossed) {
      return {
        ...defaultFingersCrossedConfig,
        ...options.fingersCrossed
      };
    }
    if (!options.fingersCrossedEnabled) {
      return null;
    }
    if (!options.fingersCrossed) {
      return { ...defaultFingersCrossedConfig };
    }
    return {
      ...defaultFingersCrossedConfig,
      ...options.fingersCrossed
    };
  }
  normalizeOptions(options) {
    const defaultOptions = {
      format: "json",
      level: "info",
      logDirectory: config2.logDirectory,
      rotation: undefined,
      timestamp: undefined,
      fingersCrossed: {},
      enabled: true,
      showTags: false,
      formatter: undefined
    };
    const mergedOptions = {
      ...defaultOptions,
      ...Object.fromEntries(Object.entries(options).filter(([, value]) => value !== undefined))
    };
    if (!mergedOptions.level || !["debug", "info", "success", "warning", "error"].includes(mergedOptions.level)) {
      mergedOptions.level = defaultOptions.level;
    }
    return mergedOptions;
  }
  async writeToFile(data) {
    const cancelled = false;
    const operationPromise = (async () => {
      let fd;
      try {
        try {
          try {
            await access(this.config.logDirectory, constants.F_OK | constants.W_OK);
          } catch {
            await mkdir(this.config.logDirectory, { recursive: true, mode: 493 });
          }
        } catch (err) {
          console.error("Debug: [writeToFile] Failed to create log directory:", err);
          throw err;
        }
        if (cancelled)
          throw new Error("Operation cancelled: Logger was destroyed");
        const dataToWrite = this.validateEncryptionConfig() ? (await this.encrypt(data)).encrypted : Buffer.from(data);
        try {
          if (!existsSync22(this.currentLogFile)) {
            await writeFile(this.currentLogFile, "", { mode: 420 });
          }
          fd = openSync(this.currentLogFile, "a", 420);
          writeFileSync22(fd, dataToWrite, { flag: "a" });
          fsyncSync(fd);
          if (fd !== undefined) {
            closeSync(fd);
            fd = undefined;
          }
          const stats = await stat(this.currentLogFile);
          if (stats.size === 0) {
            await writeFile(this.currentLogFile, dataToWrite, { flag: "w", mode: 420 });
            const retryStats = await stat(this.currentLogFile);
            if (retryStats.size === 0) {
              throw new Error("File exists but is empty after retry write");
            }
          }
        } catch (err) {
          console.error("Debug: [writeToFile] Error writing to file:", err);
          throw err;
        } finally {
          if (fd !== undefined) {
            try {
              closeSync(fd);
            } catch (err) {
              console.error("Debug: [writeToFile] Error closing file descriptor:", err);
            }
          }
        }
      } catch (err) {
        console.error("Debug: [writeToFile] Error in writeToFile:", err);
        throw err;
      }
    })();
    this.pendingOperations.push(operationPromise);
    const index = this.pendingOperations.length - 1;
    try {
      await operationPromise;
    } catch (err) {
      console.error("Debug: [writeToFile] Error in operation:", err);
      throw err;
    } finally {
      this.pendingOperations.splice(index, 1);
    }
  }
  generateLogFilename() {
    if (this.name.includes("stream-throughput") || this.name.includes("decompress-perf-test") || this.name.includes("decompression-latency") || this.name.includes("concurrent-read-test") || this.name.includes("clock-change-test")) {
      return join2(this.config.logDirectory, `${this.name}.log`);
    }
    if (this.name.includes("pending-test") || this.name.includes("temp-file-test") || this.name === "crash-test" || this.name === "corrupt-test" || this.name.includes("rotation-load-test") || this.name === "sigterm-test" || this.name === "sigint-test" || this.name === "failed-rotation-test" || this.name === "integration-test") {
      return join2(this.config.logDirectory, `${this.name}.log`);
    }
    const date = new Date().toISOString().split("T")[0];
    return join2(this.config.logDirectory, `${this.name}-${date}.log`);
  }
  setupRotation() {
    if (isBrowserProcess())
      return;
    if (typeof this.config.rotation === "boolean")
      return;
    const config22 = this.config.rotation;
    let interval;
    switch (config22.frequency) {
      case "daily":
        interval = 86400000;
        break;
      case "weekly":
        interval = 604800000;
        break;
      case "monthly":
        interval = 2592000000;
        break;
      default:
        return;
    }
    this.rotationTimeout = setInterval(() => {
      this.rotateLog();
    }, interval);
  }
  setupKeyRotation() {
    if (!this.validateEncryptionConfig()) {
      console.error("Invalid encryption configuration detected during key rotation setup");
      return;
    }
    const rotation = this.config.rotation;
    const keyRotation = rotation.keyRotation;
    if (!keyRotation?.enabled) {
      return;
    }
    const rotationInterval = typeof keyRotation.interval === "number" ? keyRotation.interval : 60;
    const interval = Math.max(rotationInterval, 60) * 1000;
    this.keyRotationTimeout = setInterval(() => {
      this.rotateKeys().catch((error) => {
        console.error("Error rotating keys:", error);
      });
    }, interval);
  }
  async rotateKeys() {
    if (!this.validateEncryptionConfig()) {
      console.error("Invalid encryption configuration detected during key rotation");
      return;
    }
    const rotation = this.config.rotation;
    const keyRotation = rotation.keyRotation;
    const newKeyId = this.generateKeyId();
    const newKey = this.generateKey();
    this.currentKeyId = newKeyId;
    this.keys.set(newKeyId, newKey);
    this.encryptionKeys.set(newKeyId, {
      key: newKey,
      createdAt: new Date
    });
    const sortedKeys = Array.from(this.encryptionKeys.entries()).sort(([, a], [, b]) => b.createdAt.getTime() - a.createdAt.getTime());
    const maxKeyCount = typeof keyRotation.maxKeys === "number" ? keyRotation.maxKeys : 1;
    const maxKeys = Math.max(1, maxKeyCount);
    if (sortedKeys.length > maxKeys) {
      for (const [keyId] of sortedKeys.slice(maxKeys)) {
        this.encryptionKeys.delete(keyId);
        this.keys.delete(keyId);
      }
    }
  }
  generateKeyId() {
    return randomBytes(16).toString("hex");
  }
  generateKey() {
    return randomBytes(32);
  }
  getCurrentKey() {
    if (!this.currentKeyId) {
      throw new Error("Encryption is not properly initialized. Make sure encryption is enabled in the configuration.");
    }
    const key = this.keys.get(this.currentKeyId);
    if (!key) {
      throw new Error(`No key found for ID ${this.currentKeyId}. The encryption key may have been rotated or removed.`);
    }
    return { key, id: this.currentKeyId };
  }
  encrypt(data) {
    const { key } = this.getCurrentKey();
    const iv = randomBytes(16);
    const cipher = createCipheriv("aes-256-gcm", key, iv);
    const encrypted = Buffer.concat([
      cipher.update(data, "utf8"),
      cipher.final()
    ]);
    const authTag = cipher.getAuthTag();
    return {
      encrypted: Buffer.concat([iv, encrypted, authTag]),
      iv
    };
  }
  async compressData(data) {
    return new Promise((resolve32, reject) => {
      const gzip = createGzip();
      const chunks = [];
      gzip.on("data", (chunk2) => chunks.push(chunk2));
      gzip.on("end", () => resolve32(Buffer.from(Buffer.concat(chunks))));
      gzip.on("error", reject);
      gzip.write(data);
      gzip.end();
    });
  }
  getEncryptionOptions() {
    if (!this.config.rotation || typeof this.config.rotation === "boolean" || !this.config.rotation.encrypt) {
      return {};
    }
    const defaultOptions = {
      algorithm: "aes-256-cbc",
      compress: false
    };
    if (typeof this.config.rotation.encrypt === "object") {
      const encryptConfig = this.config.rotation.encrypt;
      return {
        ...defaultOptions,
        ...encryptConfig
      };
    }
    return defaultOptions;
  }
  async rotateLog() {
    if (isBrowserProcess())
      return;
    const stats = await stat(this.currentLogFile).catch(() => null);
    if (!stats)
      return;
    const config22 = this.config.rotation;
    if (typeof config22 === "boolean")
      return;
    if (config22.maxSize && stats.size >= config22.maxSize) {
      const oldFile = this.currentLogFile;
      const newFile = this.generateLogFilename();
      if (this.name.includes("rotation-load-test") || this.name === "failed-rotation-test") {
        const files = await readdir(this.config.logDirectory);
        const rotatedFiles = files.filter((f) => f.startsWith(this.name) && /\.log\.\d+$/.test(f)).sort((a, b) => {
          const numA = Number.parseInt(a.match(/\.log\.(\d+)$/)?.[1] || "0");
          const numB = Number.parseInt(b.match(/\.log\.(\d+)$/)?.[1] || "0");
          return numB - numA;
        });
        const nextNum = rotatedFiles.length > 0 ? Number.parseInt(rotatedFiles[0].match(/\.log\.(\d+)$/)?.[1] || "0") + 1 : 1;
        const rotatedFile = `${oldFile}.${nextNum}`;
        if (await stat(oldFile).catch(() => null)) {
          try {
            await rename(oldFile, rotatedFile);
            if (config22.compress) {
              try {
                const compressedPath = `${rotatedFile}.gz`;
                await this.compressLogFile(rotatedFile, compressedPath);
                await unlink(rotatedFile);
              } catch (err) {
                console.error("Error compressing rotated file:", err);
              }
            }
            if (rotatedFiles.length === 0 && !files.some((f) => f.endsWith(".log.1"))) {
              try {
                const backupPath = `${oldFile}.1`;
                await writeFile(backupPath, "");
              } catch (err) {
                console.error("Error creating backup file:", err);
              }
            }
          } catch (err) {
            console.error(`Error during rotation: ${err instanceof Error ? err.message : String(err)}`);
          }
        }
      } else {
        const timestamp = new Date().toISOString().replace(/[:.]/g, "-");
        const rotatedFile = oldFile.replace(/\.log$/, `-${timestamp}.log`);
        if (await stat(oldFile).catch(() => null)) {
          await rename(oldFile, rotatedFile);
        }
      }
      this.currentLogFile = newFile;
      if (config22.maxFiles) {
        const files = await readdir(this.config.logDirectory);
        const logFiles = files.filter((f) => f.startsWith(this.name)).sort((a, b) => b.localeCompare(a));
        for (const file of logFiles.slice(config22.maxFiles)) {
          await unlink(join2(this.config.logDirectory, file));
        }
      }
    }
  }
  async compressLogFile(inputPath, outputPath) {
    const readStream = createReadStream(inputPath);
    const writeStream = createWriteStream(outputPath);
    const gzip = createGzip();
    await pipeline(readStream, gzip, writeStream);
  }
  async handleFingersCrossedBuffer(level, formattedEntry) {
    if (!this.fingersCrossedConfig)
      return;
    if (this.shouldActivateFingersCrossed(level) && !this.isActivated) {
      this.isActivated = true;
      for (const entry of this.logBuffer) {
        const formattedBufferedEntry = await this.formatter.format(entry);
        await this.writeToFile(formattedBufferedEntry);
        console.log(formattedBufferedEntry);
      }
      if (this.fingersCrossedConfig.stopBuffering)
        this.logBuffer = [];
    }
    if (this.isActivated) {
      await this.writeToFile(formattedEntry);
      console.log(formattedEntry);
    } else {
      if (this.logBuffer.length >= this.fingersCrossedConfig.bufferSize)
        this.logBuffer.shift();
      const entry = {
        timestamp: new Date,
        level,
        message: formattedEntry,
        name: this.name
      };
      this.logBuffer.push(entry);
    }
  }
  shouldActivateFingersCrossed(level) {
    if (!this.fingersCrossedConfig)
      return false;
    return this.getLevelValue(level) >= this.getLevelValue(this.fingersCrossedConfig.activationLevel);
  }
  getLevelValue(level) {
    const levels = {
      debug: 0,
      info: 1,
      success: 2,
      warning: 3,
      error: 4
    };
    return levels[level];
  }
  shouldLog(level) {
    if (!this.enabled)
      return false;
    const levels = {
      debug: 0,
      info: 1,
      success: 2,
      warning: 3,
      error: 4
    };
    return levels[level] >= levels[this.config.level];
  }
  async flushPendingWrites() {
    await Promise.all(this.pendingOperations.map((op) => {
      if (op instanceof Promise) {
        return op.catch((err) => {
          console.error("Error in pending write operation:", err);
        });
      }
      return Promise.resolve();
    }));
    if (existsSync22(this.currentLogFile)) {
      try {
        const fd = openSync(this.currentLogFile, "r+");
        fsyncSync(fd);
        closeSync(fd);
      } catch (error) {
        console.error(`Error flushing file: ${error}`);
      }
    }
  }
  async destroy() {
    if (this.rotationTimeout)
      clearInterval(this.rotationTimeout);
    if (this.keyRotationTimeout)
      clearInterval(this.keyRotationTimeout);
    this.timers.clear();
    for (const op of this.pendingOperations) {
      if (typeof op.cancel === "function") {
        op.cancel();
      }
    }
    return (async () => {
      if (this.pendingOperations.length > 0) {
        try {
          await Promise.allSettled(this.pendingOperations);
        } catch (err) {
          console.error("Error waiting for pending operations:", err);
        }
      }
      if (!isBrowserProcess() && this.config.rotation && typeof this.config.rotation !== "boolean" && this.config.rotation.compress) {
        try {
          const files = await readdir(this.config.logDirectory);
          const tempFiles = files.filter((f) => (f.includes("temp") || f.includes(".tmp")) && f.includes(this.name));
          for (const tempFile of tempFiles) {
            try {
              await unlink(join2(this.config.logDirectory, tempFile));
            } catch (err) {
              console.error(`Failed to delete temp file ${tempFile}:`, err);
            }
          }
        } catch (err) {
          console.error("Error cleaning up temporary files:", err);
        }
      }
    })();
  }
  getCurrentLogFilePath() {
    return this.currentLogFile;
  }
  formatTag(name) {
    if (!name)
      return "";
    return `${this.tagFormat.prefix}${name}${this.tagFormat.suffix}`;
  }
  formatFileTimestamp(date) {
    return `[${date.toISOString()}]`;
  }
  formatConsoleTimestamp(date) {
    return this.fancy ? terminalStyles.gray(date.toLocaleTimeString()) : date.toLocaleTimeString();
  }
  formatConsoleMessage(parts) {
    const { timestamp, icon = "", tag = "", message, level, showTimestamp = true } = parts;
    const stripAnsi = (str) => str.replace(this.ANSI_PATTERN, "");
    if (!this.fancy) {
      const components = [];
      if (showTimestamp)
        components.push(timestamp);
      if (level === "warning")
        components.push("WARN");
      else if (level === "error")
        components.push("ERROR");
      else if (icon)
        components.push(icon.replace(/[^\p{L}\p{N}\p{P}\p{Z}]/gu, ""));
      if (tag)
        components.push(tag.replace(/[[\]]/g, ""));
      components.push(message);
      return components.join(" ");
    }
    const terminalWidth = process5.stdout.columns || 120;
    let mainPart = "";
    if (level === "warning" || level === "error") {
      mainPart = `${icon} ${message}`;
    } else if (level === "info" || level === "success") {
      mainPart = `${icon} ${tag} ${message}`;
    } else {
      mainPart = `${icon} ${tag} ${terminalStyles.cyan(message)}`;
    }
    if (!showTimestamp) {
      return mainPart.trim();
    }
    const visibleMainPartLength = stripAnsi(mainPart).trim().length;
    const visibleTimestampLength = stripAnsi(timestamp).length;
    const padding = Math.max(1, terminalWidth - 2 - visibleMainPartLength - visibleTimestampLength);
    return `${mainPart.trim()}${" ".repeat(padding)}${timestamp}`;
  }
  formatMessage(message, args) {
    if (args.length === 1 && Array.isArray(args[0])) {
      return message.replace(/\{(\d+)\}/g, (match, index) => {
        const position = Number.parseInt(index, 10);
        return position < args[0].length ? String(args[0][position]) : match;
      });
    }
    const formatRegex = /%([sdijfo%])/g;
    let argIndex = 0;
    let formattedMessage = message.replace(formatRegex, (match, type) => {
      if (type === "%")
        return "%";
      if (argIndex >= args.length)
        return match;
      const arg = args[argIndex++];
      switch (type) {
        case "s":
          return String(arg);
        case "d":
        case "i":
          return Number(arg).toString();
        case "j":
        case "o":
          return JSON.stringify(arg, null, 2);
        default:
          return match;
      }
    });
    if (argIndex < args.length) {
      formattedMessage += ` ${args.slice(argIndex).map((arg) => typeof arg === "object" ? JSON.stringify(arg, null, 2) : String(arg)).join(" ")}`;
    }
    return formattedMessage;
  }
  async log(level, message, ...args) {
    if (!this.shouldLog(level))
      return;
    const timestamp = new Date;
    const consoleTime = this.formatConsoleTimestamp(timestamp);
    const fileTime = this.formatFileTimestamp(timestamp);
    let formattedMessage;
    let errorStack;
    if (message instanceof Error) {
      formattedMessage = message.message;
      errorStack = message.stack;
    } else {
      formattedMessage = this.formatMessage(message, args);
    }
    if (this.fancy && !isBrowserProcess()) {
      const icon = levelIcons[level];
      const tag = this.options.showTags !== false && this.name ? terminalStyles.gray(this.formatTag(this.name)) : "";
      let consoleMessage;
      switch (level) {
        case "debug":
          consoleMessage = this.formatConsoleMessage({
            timestamp: consoleTime,
            icon,
            tag,
            message: terminalStyles.gray(formattedMessage),
            level
          });
          console.error(consoleMessage);
          break;
        case "info":
          consoleMessage = this.formatConsoleMessage({
            timestamp: consoleTime,
            icon,
            tag,
            message: formattedMessage,
            level
          });
          console.error(consoleMessage);
          break;
        case "success":
          consoleMessage = this.formatConsoleMessage({
            timestamp: consoleTime,
            icon,
            tag,
            message: terminalStyles.green(formattedMessage),
            level
          });
          console.error(consoleMessage);
          break;
        case "warning":
          consoleMessage = this.formatConsoleMessage({
            timestamp: consoleTime,
            icon,
            tag,
            message: formattedMessage,
            level
          });
          console.warn(consoleMessage);
          break;
        case "error":
          consoleMessage = this.formatConsoleMessage({
            timestamp: consoleTime,
            icon,
            tag,
            message: formattedMessage,
            level
          });
          console.error(consoleMessage);
          if (errorStack) {
            const stackLines = errorStack.split(`
`);
            for (const line of stackLines) {
              if (line.trim() && !line.includes(formattedMessage)) {
                console.error(this.formatConsoleMessage({
                  timestamp: consoleTime,
                  message: terminalStyles.gray(`  ${line}`),
                  level,
                  showTimestamp: false
                }));
              }
            }
          }
          break;
      }
    } else if (!isBrowserProcess()) {
      console.error(`${fileTime} ${level.toUpperCase()}: ${formattedMessage}`);
      if (errorStack) {
        console.error(errorStack);
      }
    }
    let logEntry = `${fileTime} local.${level.toUpperCase()}: ${formattedMessage}
`;
    if (errorStack) {
      logEntry += `${errorStack}
`;
    }
    await this.writeToFile(logEntry);
  }
  time(label) {
    const start = performance.now();
    if (this.fancy && !isBrowserProcess()) {
      const tag = this.options.showTags !== false && this.name ? terminalStyles.gray(this.formatTag(this.name)) : "";
      const consoleTime = this.formatConsoleTimestamp(new Date);
      console.error(this.formatConsoleMessage({
        timestamp: consoleTime,
        icon: terminalStyles.blue("◐"),
        tag,
        message: `${terminalStyles.cyan(label)}...`
      }));
    }
    return async (metadata) => {
      if (!this.enabled)
        return;
      const end = performance.now();
      const elapsed = Math.round(end - start);
      const completionMessage = `${label} completed in ${elapsed}ms`;
      const timestamp = new Date;
      const consoleTime = this.formatConsoleTimestamp(timestamp);
      const fileTime = this.formatFileTimestamp(timestamp);
      let logEntry = `${fileTime} local.INFO: ${completionMessage}`;
      if (metadata) {
        logEntry += ` ${JSON.stringify(metadata)}`;
      }
      logEntry += `
`;
      if (this.fancy && !isBrowserProcess()) {
        const tag = this.options.showTags !== false && this.name ? terminalStyles.gray(this.formatTag(this.name)) : "";
        console.error(this.formatConsoleMessage({
          timestamp: consoleTime,
          icon: terminalStyles.green("✓"),
          tag,
          message: `${completionMessage}${metadata ? ` ${JSON.stringify(metadata)}` : ""}`
        }));
      } else if (!isBrowserProcess()) {
        console.error(logEntry.trim());
      }
      await this.writeToFile(logEntry);
    };
  }
  async debug(message, ...args) {
    await this.log("debug", message, ...args);
  }
  async info(message, ...args) {
    await this.log("info", message, ...args);
  }
  async success(message, ...args) {
    await this.log("success", message, ...args);
  }
  async warn(message, ...args) {
    await this.log("warning", message, ...args);
  }
  async error(message, ...args) {
    await this.log("error", message, ...args);
  }
  validateEncryptionConfig() {
    if (!this.config.rotation)
      return false;
    if (typeof this.config.rotation === "boolean")
      return false;
    const rotation = this.config.rotation;
    const { encrypt } = rotation;
    return !!encrypt;
  }
  async only(fn) {
    if (!this.enabled)
      return;
    return await fn();
  }
  isEnabled() {
    return this.enabled;
  }
  setEnabled(enabled) {
    this.enabled = enabled;
  }
  extend(namespace) {
    const childName = `${this.name}:${namespace}`;
    const childLogger = new Logger(childName, {
      ...this.options,
      logDirectory: this.config.logDirectory,
      level: this.config.level,
      format: this.config.format,
      rotation: typeof this.config.rotation === "boolean" ? undefined : this.config.rotation,
      timestamp: typeof this.config.timestamp === "boolean" ? undefined : this.config.timestamp
    });
    this.subLoggers.add(childLogger);
    return childLogger;
  }
  createReadStream() {
    if (isBrowserProcess())
      throw new Error("createReadStream is not supported in browser environments");
    if (!existsSync22(this.currentLogFile))
      throw new Error(`Log file does not exist: ${this.currentLogFile}`);
    return createReadStream(this.currentLogFile, { encoding: "utf8" });
  }
  async decrypt(data) {
    if (!this.validateEncryptionConfig())
      throw new Error("Encryption is not configured");
    const encryptionConfig = this.config.rotation;
    if (!encryptionConfig.encrypt || typeof encryptionConfig.encrypt === "boolean")
      throw new Error("Invalid encryption configuration");
    if (!this.currentKeyId || !this.keys.has(this.currentKeyId))
      throw new Error("No valid encryption key available");
    const key = this.keys.get(this.currentKeyId);
    try {
      const encryptedData = Buffer.isBuffer(data) ? data : Buffer.from(data, "base64");
      const iv = encryptedData.slice(0, 16);
      const authTag = encryptedData.slice(-16);
      const ciphertext = encryptedData.slice(16, -16);
      const decipher = createDecipheriv("aes-256-gcm", key, iv);
      decipher.setAuthTag(authTag);
      const decrypted = Buffer.concat([
        decipher.update(ciphertext),
        decipher.final()
      ]);
      return decrypted.toString("utf8");
    } catch (err) {
      throw new Error(`Decryption failed: ${err instanceof Error ? err.message : String(err)}`);
    }
  }
  getLevel() {
    return this.config.level;
  }
  getLogDirectory() {
    return this.config.logDirectory;
  }
  getFormat() {
    return this.config.format;
  }
  getRotationConfig() {
    return this.config.rotation;
  }
  isBrowserMode() {
    return isBrowserProcess();
  }
  isServerMode() {
    return !isBrowserProcess();
  }
  setTestEncryptionKey(keyId, key) {
    this.currentKeyId = keyId;
    this.keys.set(keyId, key);
  }
  getTestCurrentKey() {
    if (!this.currentKeyId || !this.keys.has(this.currentKeyId)) {
      return null;
    }
    return {
      id: this.currentKeyId,
      key: this.keys.get(this.currentKeyId)
    };
  }
  getConfig() {
    return this.config;
  }
  async box(message) {
    if (!this.enabled)
      return;
    const timestamp = new Date;
    const consoleTime = this.formatConsoleTimestamp(timestamp);
    const fileTime = this.formatFileTimestamp(timestamp);
    if (this.fancy && !isBrowserProcess()) {
      const lines = message.split(`
`);
      const width = Math.max(...lines.map((line) => line.length)) + 2;
      const top = `┌${"─".repeat(width)}┐`;
      const bottom = `└${"─".repeat(width)}┘`;
      const boxedLines = lines.map((line) => {
        const padding = " ".repeat(width - line.length - 2);
        return `│ ${line}${padding} │`;
      });
      if (this.options.showTags !== false && this.name) {
        console.error(this.formatConsoleMessage({
          timestamp: consoleTime,
          message: terminalStyles.gray(this.formatTag(this.name)),
          showTimestamp: false
        }));
      }
      console.error(this.formatConsoleMessage({
        timestamp: consoleTime,
        message: terminalStyles.cyan(top)
      }));
      boxedLines.forEach((line) => console.error(this.formatConsoleMessage({
        timestamp: consoleTime,
        message: terminalStyles.cyan(line),
        showTimestamp: false
      })));
      console.error(this.formatConsoleMessage({
        timestamp: consoleTime,
        message: terminalStyles.cyan(bottom),
        showTimestamp: false
      }));
    } else if (!isBrowserProcess()) {
      console.error(`${fileTime} BOX: ${message}`);
    }
    const logEntry = `${fileTime} local.INFO: [BOX] ${message}
`;
    await this.writeToFile(logEntry);
  }
  async prompt(message) {
    if (isBrowserProcess()) {
      return Promise.resolve(true);
    }
    return new Promise((resolve32) => {
      console.error(`${terminalStyles.cyan("?")} ${message} (y/n) `);
      const onData = (data) => {
        const input = data.toString().trim().toLowerCase();
        process5.stdin.removeListener("data", onData);
        try {
          if (typeof process5.stdin.setRawMode === "function") {
            process5.stdin.setRawMode(false);
          }
        } catch {}
        process5.stdin.pause();
        console.error("");
        resolve32(input === "y" || input === "yes");
      };
      try {
        if (typeof process5.stdin.setRawMode === "function") {
          process5.stdin.setRawMode(true);
        }
      } catch {}
      process5.stdin.resume();
      process5.stdin.once("data", onData);
    });
  }
  setFancy(enabled) {
    this.fancy = enabled;
  }
  isFancy() {
    return this.fancy;
  }
  pause() {
    this.enabled = false;
  }
  resume() {
    this.enabled = true;
  }
  async start(message, ...args) {
    if (!this.enabled)
      return;
    let formattedMessage = message;
    if (args && args.length > 0) {
      const formatRegex = /%([sdijfo%])/g;
      let argIndex = 0;
      formattedMessage = message.replace(formatRegex, (match, type) => {
        if (type === "%")
          return "%";
        if (argIndex >= args.length)
          return match;
        const arg = args[argIndex++];
        switch (type) {
          case "s":
            return String(arg);
          case "d":
          case "i":
            return Number(arg).toString();
          case "j":
          case "o":
            return JSON.stringify(arg, null, 2);
          default:
            return match;
        }
      });
      if (argIndex < args.length) {
        formattedMessage += ` ${args.slice(argIndex).map((arg) => typeof arg === "object" ? JSON.stringify(arg, null, 2) : String(arg)).join(" ")}`;
      }
    }
    if (this.fancy && !isBrowserProcess()) {
      const tag = this.options.showTags !== false && this.name ? terminalStyles.gray(this.formatTag(this.name)) : "";
      const spinnerChar = terminalStyles.blue("◐");
      console.error(`${spinnerChar} ${tag} ${terminalStyles.cyan(formattedMessage)}`);
    }
    const timestamp = new Date;
    const formattedDate = timestamp.toISOString();
    const logEntry = `[${formattedDate}] local.INFO: [START] ${formattedMessage}
`;
    await this.writeToFile(logEntry);
  }
  progress(total, initialMessage = "") {
    if (!this.enabled || !this.fancy || isBrowserProcess() || total <= 0) {
      return {
        update: () => {},
        finish: () => {},
        interrupt: () => {}
      };
    }
    if (this.activeProgressBar) {
      console.warn("Warning: Another progress bar is already active. Finishing the previous one.");
      this.finishProgressBar(this.activeProgressBar, "[Auto-finished]");
    }
    const barLength = 20;
    this.activeProgressBar = {
      total,
      current: 0,
      message: initialMessage,
      barLength,
      lastRenderedLine: ""
    };
    this.renderProgressBar(this.activeProgressBar);
    const update = (current, message) => {
      if (!this.activeProgressBar || !this.enabled || !this.fancy || isBrowserProcess())
        return;
      this.activeProgressBar.current = Math.max(0, Math.min(total, current));
      if (message !== undefined) {
        this.activeProgressBar.message = message;
      }
      this.renderProgressBar(this.activeProgressBar);
    };
    const finish = (message) => {
      if (!this.activeProgressBar || !this.enabled || !this.fancy || isBrowserProcess())
        return;
      this.activeProgressBar.current = this.activeProgressBar.total;
      if (message !== undefined) {
        this.activeProgressBar.message = message;
      }
      this.renderProgressBar(this.activeProgressBar, true);
      this.finishProgressBar(this.activeProgressBar);
    };
    const interrupt = (interruptMessage, level = "info") => {
      if (!this.activeProgressBar || !this.enabled || !this.fancy || isBrowserProcess())
        return;
      process5.stdout.write(`${"\r".padEnd(process5.stdout.columns || 80)}\r`);
      this.log(level, interruptMessage);
      setTimeout(() => {
        if (this.activeProgressBar) {
          this.renderProgressBar(this.activeProgressBar);
        }
      }, 50);
    };
    return { update, finish, interrupt };
  }
  renderProgressBar(barState, isFinished = false) {
    if (!this.enabled || !this.fancy || isBrowserProcess() || !process5.stdout.isTTY)
      return;
    const percent = Math.min(100, Math.max(0, Math.round(barState.current / barState.total * 100)));
    const filledLength = Math.round(barState.barLength * percent / 100);
    const emptyLength = barState.barLength - filledLength;
    const filledBar = terminalStyles.green("━".repeat(filledLength));
    const emptyBar = terminalStyles.gray("━".repeat(emptyLength));
    const bar = `[${filledBar}${emptyBar}]`;
    const percentageText = `${percent}%`.padStart(4);
    const messageText = barState.message ? ` ${barState.message}` : "";
    const icon = isFinished ? terminalStyles.green("✓") : terminalStyles.blue("▶");
    const line = `\r${icon} ${bar} ${percentageText}${messageText}`;
    const terminalWidth = process5.stdout.columns || 80;
    const clearLine = " ".repeat(Math.max(0, terminalWidth - line.replace(this.ANSI_PATTERN, "").length));
    barState.lastRenderedLine = `${line}${clearLine}`;
    process5.stdout.write(barState.lastRenderedLine);
    if (isFinished) {
      process5.stdout.write(`
`);
    }
  }
  finishProgressBar(barState, finalMessage) {
    if (!this.enabled || !this.fancy || isBrowserProcess() || !process5.stdout.isTTY) {
      this.activeProgressBar = null;
      return;
    }
    if (barState.current < barState.total) {
      barState.current = barState.total;
    }
    if (finalMessage)
      barState.message = finalMessage;
    this.renderProgressBar(barState, true);
    this.activeProgressBar = null;
  }
}
var logger = new Logger("stacks");

// src/utils/logger.ts
var logger2 = new Logger("aax", {
  fancy: true,
  level: "info",
  showTags: true
});
function parseFFmpegProgress(output) {
  const result = {};
  const timeMatch = output.match(/time=\s*(\d+):(\d+):(\d+)(\.(\d+))?/i);
  if (timeMatch) {
    const hours = Number.parseInt(timeMatch[1], 10);
    const minutes = Number.parseInt(timeMatch[2], 10);
    const seconds = Number.parseInt(timeMatch[3], 10);
    const milliseconds = timeMatch[5] ? Number.parseInt(timeMatch[5], 10) : 0;
    result.timeMs = hours * 3600000 + minutes * 60000 + seconds * 1000 + milliseconds;
  }
  const durationMatch = output.match(/Duration:\s*(\d+):(\d+):(\d+)(\.(\d+))?/i);
  if (durationMatch) {
    const hours = Number.parseInt(durationMatch[1], 10);
    const minutes = Number.parseInt(durationMatch[2], 10);
    const seconds = Number.parseInt(durationMatch[3], 10);
    const milliseconds = durationMatch[5] ? Number.parseInt(durationMatch[5], 10) : 0;
    result.totalMs = hours * 3600000 + minutes * 60000 + seconds * 1000 + milliseconds;
  }
  if (result.timeMs !== undefined && result.totalMs !== undefined && result.totalMs > 0) {
    result.progress = Math.min(100, result.timeMs / result.totalMs * 100);
  }
  const speedMatch = output.match(/speed=\s*(\S+)x/i);
  if (speedMatch) {
    result.speed = speedMatch[1];
  }
  const sizeMatch = output.match(/size=\s*(\S+)/i);
  if (sizeMatch) {
    result.size = sizeMatch[1];
  }
  return result;
}
function formatTime(ms) {
  const seconds = Math.floor(ms / 1000);
  const hours = Math.floor(seconds / 3600);
  const minutes = Math.floor(seconds % 3600 / 60);
  const secs = seconds % 60;
  return `${hours.toString().padStart(2, "0")}:${minutes.toString().padStart(2, "0")}:${secs.toString().padStart(2, "0")}`;
}

// src/utils/ffmpeg.ts
var supportedOptions = new Map;
async function checkFFmpeg() {
  try {
    const ffmpegPath = config.ffmpegPath || "ffmpeg";
    if (config.ffmpegPath && !existsSync4(config.ffmpegPath)) {
      logger2.error(`Configured FFmpeg path does not exist: ${config.ffmpegPath}`);
      return false;
    }
    execSync2(`${ffmpegPath} -version`, { stdio: "ignore" });
    return true;
  } catch {
    return false;
  }
}
async function runFFmpeg(args) {
  const isMetadataExtraction = args.includes("-f") && args.includes("ffmetadata");
  return new Promise((resolve4) => {
    const ffmpegPath = config.ffmpegPath || "ffmpeg";
    const ffmpeg = spawn(ffmpegPath, args);
    let output = "";
    let bookDuration;
    let currentTimeMs = 0;
    let lastOutputTime = Date.now();
    let isFirstProgress = true;
    let currentSize = "0KB";
    let currentSpeed = "0x";
    const progressBar = isMetadataExtraction ? null : logger2.progress(100, "Preparing conversion...");
    const updateInterval = setInterval(() => {
      if (!progressBar || !bookDuration)
        return;
      const now = Date.now();
      if (now - lastOutputTime > 3000 && currentTimeMs > 0) {
        const progressPercent = Math.max(0.1, Math.min(99.9, currentTimeMs / bookDuration * 100));
        progressBar.update(progressPercent, `Converting ${formatTime(currentTimeMs)} / ${formatTime(bookDuration)} (${currentSpeed}) - Size: ${currentSize}`);
        currentTimeMs += 100;
      }
    }, 1000);
    function updateProgressFromOutput(text) {
      if (!progressBar)
        return;
      const progress = parseFFmpegProgress(text);
      if (progress.totalMs && progress.totalMs > 0 && !bookDuration) {
        bookDuration = progress.totalMs;
      }
      if (progress.timeMs !== undefined) {
        currentTimeMs = progress.timeMs;
        lastOutputTime = Date.now();
        if (bookDuration) {
          const progressPercent = Math.max(0.1, Math.min(99.9, currentTimeMs / bookDuration * 100));
          if (progress.size)
            currentSize = progress.size;
          if (progress.speed)
            currentSpeed = progress.speed;
          let message = `Converting ${formatTime(currentTimeMs)} / ${formatTime(bookDuration)}`;
          if (progress.speed)
            message += ` (${progress.speed}x)`;
          if (progress.size)
            message += ` - Size: ${progress.size}`;
          progressBar.update(progressPercent, message);
          if (isFirstProgress) {
            logger2.debug(`First progress update: ${progressPercent.toFixed(1)}%, time=${formatTime(currentTimeMs)}`);
            isFirstProgress = false;
          }
        }
      }
    }
    ffmpeg.stdout.on("data", (data) => {
      const chunk = data.toString();
      output += chunk;
      if (!isMetadataExtraction) {
        updateProgressFromOutput(chunk);
      }
    });
    let stderrBuffer = "";
    ffmpeg.stderr.on("data", (data) => {
      const chunk = data.toString();
      output += chunk;
      if (!isMetadataExtraction) {
        stderrBuffer += chunk;
        if (stderrBuffer.includes("time=") || stderrBuffer.includes("Duration:")) {
          updateProgressFromOutput(stderrBuffer);
          if (stderrBuffer.length > 1000) {
            stderrBuffer = "";
          }
        }
      }
      if (config.verbose) {
        logger2.debug(chunk.trim());
      }
    });
    ffmpeg.on("close", (code) => {
      clearInterval(updateInterval);
      if (progressBar) {
        if (code === 0) {
          progressBar.finish("Conversion completed successfully!");
        } else {
          progressBar.interrupt("Conversion failed", "error");
        }
      }
      resolve4({
        success: code === 0,
        output
      });
    });
  });
}
async function extractAAXMetadata(filePath) {
  logger2.info(`Extracting metadata from ${filePath}...`);
  const startTime = Date.now();
  const args = [
    "-i",
    filePath,
    "-f",
    "ffmetadata"
  ];
  const { success, output } = await runFFmpeg(args);
  const duration = Date.now() - startTime;
  logger2.debug(`Metadata extraction completed in ${duration}ms`);
  return success ? output : "";
}

// src/utils/metadata.ts
function parseMetadata(metadataText) {
  const metadata = {};
  const lines = metadataText.split(`
`);
  for (const line of lines) {
    if (line.startsWith("title=")) {
      metadata.title = line.substring(6).trim();
    } else if (line.startsWith("artist=")) {
      metadata.author = line.substring(7).trim();
    } else if (line.startsWith("album_artist=")) {
      metadata.narrator = line.substring(13).trim();
    }
  }
  return metadata;
}
function extractChapters(metadataText) {
  const chapters = [];
  const lines = metadataText.split(`
`);
  let inChapter = false;
  let currentChapter = {};
  for (const line of lines) {
    if (line === "[CHAPTER]") {
      inChapter = true;
      currentChapter = {};
    } else if (line === "[/CHAPTER]") {
      inChapter = false;
      if (currentChapter.title && currentChapter.startTime !== undefined && currentChapter.endTime !== undefined) {
        chapters.push(currentChapter);
      }
    } else if (inChapter) {
      if (line.startsWith("TITLE=")) {
        currentChapter.title = line.substring(6).trim();
      } else if (line.startsWith("START=")) {
        currentChapter.startTime = Number.parseFloat(line.substring(6)) / 1000;
      } else if (line.startsWith("END=")) {
        currentChapter.endTime = Number.parseFloat(line.substring(4)) / 1000;
      }
    }
  }
  return chapters;
}
async function getBookMetadata(filePath, existingMetadata) {
  const metadataText = existingMetadata || await extractAAXMetadata(filePath);
  const basicMetadata = parseMetadata(metadataText);
  const chapters = extractChapters(metadataText);
  return {
    ...basicMetadata,
    chapters
  };
}

// src/converter.ts
function generateOutputPath(metadata, options) {
  const outputDir = options.outputDir || config.outputDir || ".";
  const outputFormat = options.outputFormat || config.outputFormat || "mp3";
  let basePath = outputDir;
  if (!options.flatFolderStructure) {
    if (metadata.author) {
      basePath = path2.join(basePath, metadata.author);
    }
    if (options.seriesTitleInFolderStructure && metadata.series) {
      basePath = path2.join(basePath, metadata.series);
    }
    const bookFolder = options.fullCaptionForBookFolder ? metadata.title : metadata.title?.split(":")[0];
    if (bookFolder) {
      basePath = path2.join(basePath, bookFolder);
    }
  }
  mkdirSync3(basePath, { recursive: true });
  let filename = metadata.title || path2.basename(options.inputFile, path2.extname(options.inputFile));
  if (metadata.seriesIndex && options.sequenceNumberDigits) {
    const partNum = String(metadata.seriesIndex).padStart(options.sequenceNumberDigits, "0");
    filename = `${options.partFolderPrefix || ""}${partNum} - ${filename}`;
  }
  return path2.join(basePath, `${filename}.${outputFormat}`);
}
async function convertAAX(options) {
  if (!existsSync5(options.inputFile)) {
    logger2.error(`Input file does not exist: ${options.inputFile}`);
    return {
      success: false,
      error: `Input file does not exist: ${options.inputFile}`
    };
  }
  await logger2.box(`Converting ${path2.basename(options.inputFile)}`);
  logger2.info("Checking FFmpeg installation...");
  const ffmpegAvailable = await checkFFmpeg();
  if (!ffmpegAvailable) {
    logger2.error("FFmpeg is not available. Please install FFmpeg or set ffmpegPath in config.");
    return {
      success: false,
      error: "FFmpeg is not available. Please install FFmpeg or set ffmpegPath in config."
    };
  }
  const activationCode = options.activationCode || config.activationCode;
  if (!activationCode) {
    logger2.error("No activation code provided. This is required to convert AAX files.");
    return {
      success: false,
      error: "No activation code provided. This is required to convert AAX files."
    };
  }
  logger2.info(`Using activation code: ${activationCode.substring(0, 2)}******`);
  try {
    logger2.info("Retrieving audiobook metadata...");
    const metadata = await getBookMetadata(options.inputFile);
    const outputPath = generateOutputPath(metadata, options);
    if (metadata.title) {
      logger2.info(`Title: ${metadata.title}`);
    }
    if (metadata.author) {
      logger2.info(`Author: ${metadata.author}`);
    }
    if (metadata.narrator) {
      logger2.info(`Narrator: ${metadata.narrator}`);
    }
    if (metadata.duration) {
      const hours = Math.floor(metadata.duration / 3600);
      const minutes = Math.floor(metadata.duration % 3600 / 60);
      const seconds = Math.floor(metadata.duration % 60);
      logger2.info(`Duration: ${hours}h ${minutes}m ${seconds}s`);
    }
    if (metadata.chapters?.length) {
      logger2.info(`Chapters: ${metadata.chapters.length}`);
    }
    logger2.info(`Output format: ${options.outputFormat || config.outputFormat || "mp3"}`);
    logger2.info(`Output path: ${outputPath}`);
    logger2.info("Preparing FFmpeg command...");
    const ffmpegArgs = [];
    ffmpegArgs.push("-activation_bytes", activationCode);
    ffmpegArgs.push("-i", options.inputFile);
    ffmpegArgs.push("-map", "0:0");
    if (options.variableBitRate) {
      ffmpegArgs.push("-q:a", "0");
    } else {
      const bitrate = options.bitrate || config.bitrate || 64;
      ffmpegArgs.push("-ab", `${bitrate}k`);
    }
    ffmpegArgs.push("-map_metadata", "0");
    if (options.chaptersEnabled) {
      if (options.useNamedChapters) {
        ffmpegArgs.push("-map_chapters", "0");
      }
    }
    const outputFormat = options.outputFormat || config.outputFormat || "mp3";
    if (outputFormat === "mp3") {
      ffmpegArgs.push("-codec:a", "libmp3lame");
      ffmpegArgs.push("-write_xing", "0");
      ffmpegArgs.push("-id3v2_version", "3");
      if (options.useISOLatin1) {
        ffmpegArgs.push("-id3v2_version", "3");
        ffmpegArgs.push("-metadata_header_padding", "0");
        ffmpegArgs.push("-write_id3v1", "1");
      }
    } else if (outputFormat === "m4a" || outputFormat === "m4b") {
      ffmpegArgs.push("-codec:a", "aac");
      ffmpegArgs.push("-f", "mp4");
      ffmpegArgs.push("-movflags", "+faststart");
      ffmpegArgs.push("-metadata:s:a:0", "handler=Sound");
      if (options.aacEncoding44_1) {
        ffmpegArgs.push("-ar", "44100");
      }
    }
    if (options.extractCoverImage) {
      const coverPath = path2.join(path2.dirname(outputPath), "cover.jpg");
      ffmpegArgs.push("-map", "0:v");
      ffmpegArgs.push("-c:v", "copy");
      ffmpegArgs.push(coverPath);
    }
    ffmpegArgs.push("-y");
    ffmpegArgs.push(outputPath);
    if (config.verbose) {
      logger2.debug(`FFmpeg command: ffmpeg ${ffmpegArgs.join(" ")}`);
    }
    logger2.info("Starting conversion...");
    const { success, output } = await runFFmpeg(ffmpegArgs);
    if (success) {
      logger2.success(`Conversion completed! Output saved to: ${outputPath}`);
      return {
        success: true,
        outputPath
      };
    } else {
      logger2.warn("First conversion attempt failed, trying with lowercase activation code...");
      ffmpegArgs[1] = activationCode.toLowerCase();
      if (config.verbose) {
        logger2.debug(`FFmpeg command (with lowercase code): ffmpeg ${ffmpegArgs.join(" ")}`);
      }
      const retryResult = await runFFmpeg(ffmpegArgs);
      if (retryResult.success) {
        logger2.success(`Conversion completed! Output saved to: ${outputPath}`);
        return {
          success: true,
          outputPath
        };
      }
      logger2.error("Conversion failed. Check the logs for details.");
      return {
        success: false,
        error: `FFmpeg conversion failed: ${output}`
      };
    }
  } catch (error) {
    logger2.error(`Error during conversion: ${error.message}`);
    return {
      success: false,
      error: `Error during conversion: ${error.message}`
    };
  }
}
async function splitToChapters(options) {
  logger2.info("Converting and splitting audiobook by chapters...");
  const chaptersEnabled = true;
  return convertAAX({ ...options, chaptersEnabled });
}
export {
  splitToChapters,
  isValidActivationCode,
  getActivationBytesFromAudibleCli,
  convertAAX,
  config
};
